--[[

Name: MusicStremFu
Revision: $Revision: 25931 $
Date: $Date: 2007-01-23 18:43:23 -0500 (Tue, 23 Jan 2007) $
Author(s): benumbed (someone@somewhere.com)
Website: http://www.wowace.com/wiki/MusicStream
Documentation: http://www.wowace.com/wiki/MusicStream
SVN: http://svn.wowace.com/root/trunk/MusicStream
Description: A FuBar Plugin for MusicStream.
Dependencies: Ace2

]]--

--- ========================================================= ---
--- Check to make sure MusicStream is there
--- ========================================================= ---
if not MusicStream then return end

--- ========================================================= ---
--- Check to make sure FuBarPlugin 2.0 is there
--- ========================================================= ---
if not AceLibrary:HasInstance("FuBarPlugin-2.0") then return end

--- ========================================================= ---
--- Add Locale
--- ========================================================= ---
local L = AceLibrary("AceLocale-2.2"):new("MusicStreamFu")

L:RegisterTranslations("enUS", function() return {
    ["MusicStream"] = true,
	["Toggle visibility of %s"] = true,
    ["Button Visibility"] = true,
    ["%s icon is now %s."] = true,
	["hidden"] = true,
    ["shown"] = true,
	["Menu Options"] = true,
} end)

--- ========================================================= ---
--- Add Tablet
--- ========================================================= ---
local Tablet = AceLibrary("Tablet-2.0")

--- ========================================================= ---
--- Add Music Library
--- ========================================================= ---
local Music = AceLibrary("Music-2.0")

--- ========================================================= ---
--- Register DB for FuBar
--- ========================================================= ---

MusicStreamFu = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDB-2.0", "FuBarPlugin-2.0")
MusicStreamFu.hasIcon = true
MusicStreamFu.hasNoText = true
MusicStreamFu.cannotDetachTooltip = true
MusicStreamFu.hideWithoutStandby = true
MusicStreamFu.hideMenuTitle = true
MusicStreamFu.defaultMinimapPosition = 180
MusicStreamFu.clickableTooltip = true
MusicStreamFu.OnMenuRequest = MusicStream.Options

MusicStream.hideMenuTitle = true

MusicStreamFu:RegisterDB("MusicStreamFuDB")
MusicStreamFu:RegisterDefaults("profile", {
	buttonSize = 5,
	buttonSpacing = 5,
		visible = {
		["Previous"] = true,
		["Stop"] = true,
		["Play"] = true,
		["Next"] = true,
	},
})

--- ========================================================= ---
--- Set Images Folder
--- ========================================================= ---
MusicStream_IMG = "Interface\\AddOns\\MusicStream\\Images\\"

--- ========================================================= ---
--- On Enable
--- ========================================================= ---
function MusicStreamFu:OnEnable()
	local args = AceLibrary("FuBarPlugin-2.0"):GetAceOptionsDataTable(self)
	if not MusicStreamFu.OnMenuRequest.args.settings.args.FuBarMenu then
		MusicStreamFu.OnMenuRequest.args.settings.args.FuBarMenu = {
			type = "group",
			name = L["Menu Options"],
			desc = L["Menu Options"],
			args = args,
			order = 403,
		}
	end
	
	if FuBar then
		self.fontSize = FuBar:GetFontSize()
	end
	
	self:BuildButtons()
	self.iconFrame:SetWidth(0)

end

--- ========================================================= ---
--- Set Buttons
--- ========================================================= ---

local buttons = {
	{
		id = "Stop",
		icon = MusicStream_IMG.."Stop",
		exec = function() MusicStream:Stop() end,
		key  = "TOGGLE_SCSTOP",
		name = "Stop Music",
		tip  = "Stop Playing Music",
	},
	{
		id = "Previous",
		icon = MusicStream_IMG.."Previous",
		exec = function() MusicStream:Prev() end,
		key  = "TOGGLE_SCPREV",
		name = "Previous Channel",
		tip  = "",
	},
	{
		id = "Play",
		icon = MusicStream_IMG.."Play",
		exec = function() MusicStream:Play() end,
		key  = "TOGGLE_SCPLAY",
		name = "Play Channel",
		tip  = "",
	},
	{
		id = "Next",
		icon = MusicStream_IMG.."Next",
		exec = function() MusicStream:Next() end,
		key  = "TOGGLE_SCNEXT",
		name = "Next Channel",
		tip  = "",
	},
}

--- ========================================================= ---
--- Build Buttons
--- ========================================================= ---

function MusicStreamFu:BuildButtons()
	local last
	local frameWidth = 0
	for _, data in ipairs(buttons) do
		local name = data.id
		if self:IsShowingButton(name) then
			local button_name = name
			local frame_name = "MusicStreamFuFrame" .. name
			local button = _G[frame_name] or self:CreatePluginChildFrame("Button", frame_name, self.frame)
			button:Show()
			button:EnableMouse(true)
			if last then
				button:SetPoint("LEFT", last, "RIGHT", self:GetButtonSpacing(), 0)
			else
				button:SetPoint("LEFT", self.frame, "LEFT", self:GetButtonSpacing(), 0)
			end

			local texture = _G[frame_name .. "Texture"] or button:CreateTexture(frame_name .. "Texture")
			texture:SetTexture(data.icon)
			if data.texcoord then
				texture:SetTexCoord(unpack(data.texcoord))
			end

			texture:SetAllPoints(button)

			local OnEnter = button:GetScript("OnEnter")
			button:SetScript("OnEnter", function()
				self.current_mouseover = button_name
				OnEnter()
			end)

			local OnLeave = button:GetScript("OnLeave")
			button:SetScript("OnLeave", function()
				self.current_mouseover = nil
				OnLeave()
			end)

			button:SetScript("OnClick", data.exec)
			
			if FuBar then
				button:SetWidth(FuBar:GetFontSize())
				button:SetHeight(FuBar:GetFontSize())
			end
			
			frameWidth = frameWidth + button:GetWidth() + self:GetButtonSpacing()

			last = button
		else
			local button = _G["MusicStreamFuFrame" .. name]
			if button then
				button:Hide()
			end
		end
	end

	if frameWidth > 0 then
		self:SetFontSize(self.fontSize)
	else
		self:ToggleShowingButton("Stop")
		self:BuildButtons()
	end
end

--- ========================================================= ---
--- Check if Showing Button
--- ========================================================= ---

function MusicStreamFu:IsShowingButton(button)
	return self.db.profile.visible[button]
end

--- ========================================================= ---
--- Toggle Showing Button
--- ========================================================= ---

function MusicStreamFu:ToggleShowingButton(button)
	self.db.profile.visible[button] = not self.db.profile.visible[button]
	self:CheckWidth(true)
	return self.db.profile.visible[button]
end

--- ========================================================= ---
--- Get Buttons for Visibility Toggle
--- ========================================================= ---

function MusicStreamFu:GetButtons()
	for _, data in ipairs(buttons) do
		local name = data.id
		local key_name = string.gsub(name, "%s+", "-")
		local button_name = name

		local subtbl = {}
		MusicStream.Options.args.settings.args.FuBarSettingsVisibility.args[key_name] = subtbl
		subtbl.type = "toggle"
		subtbl.name = data.name
		subtbl.desc = string.format(L["Toggle visibility of %s"], data.name)
		subtbl.get = function() return MusicStreamFu:IsShowingButton(button_name) end
		subtbl.set = function() MusicStreamFu:ToggleShowingButton(button_name) end
	end
end

--- ========================================================= ---
--- Check Width of all Visible Buttons
--- ========================================================= ---

function MusicStreamFu:CheckWidth(force)
	if force then
		self:SetFontSize(self.fontSize)
		if self.panel and self.panel:GetPluginSide(self) == "CENTER" then
			self.panel:UpdateCenteredPosition()
		end
	end
end

--- ========================================================= ---
--- Get Button Spacing
--- ========================================================= ---

function MusicStreamFu:GetButtonSpacing()
	return self.db.profile.buttonSpacing
end

--- ========================================================= ---
--- Set Button Spacing
--- ========================================================= ---

function MusicStreamFu:SetButtonSpacing(spacing)
	if spacing then
		self.db.profile.buttonSpacing = spacing
		self:SetFontSize(self.fontSize)
	end
end

--- ========================================================= ---
--- On Tooltip Update
--- ========================================================= ---

function MusicStreamFu:OnTooltipUpdate()
	if self.current_mouseover then
		local cat = Tablet:AddCategory(
			'columns', 2,
			'child_textR', 1,
			'child_textG', 1,
			'child_textB', 0
		)
		
		local data
		for _,v in ipairs(buttons) do
			if v.id == self.current_mouseover then
				data = v
				break
			end
		end
		if not data then
			return
		end
		local key = GetBindingKey(data.key)

		if key then
			key = " (" .. key .. ")"
		else
			key = ""
		end

		if (data.id == "Previous") then
			data.tip = "-------------------------";
			data.tiptitle = Music:PrevTitle();
			data.tipartist = Music:PrevArtist();
			data.tipalbum = Music:PrevAlbum();
		end
		
		if (data.id == "Play") then
			data.tip = "-------------------------";
			data.tiptitle = Music:CurrentTitle();
			data.tipartist = Music:CurrentArtist();
			data.tipalbum = Music:CurrentAlbum();
		end
		
		if (data.id == "Next") then
			data.tip = "-------------------------";
			data.tiptitle = Music:NextTitle();
			data.tipartist = Music:NextArtist();
			data.tipalbum = Music:NextAlbum();
		end
		
		Tablet:SetTitle(format("|cffffffff%s|r%s", data.name, key))
		
		cat:AddLine(
			'text', data.tip
		)
		cat:AddLine(
			'text', data.tiptitle
		)
		cat:AddLine(
			'text', data.tipartist
		)
		cat:AddLine(
			'text', data.tipalbum
		)
	end
end

--- ========================================================= ---
--- Set Font Size
--- ========================================================= ---

function MusicStreamFu:SetFontSize(fsize)
	if fsize then
		local frameWidth = 0
		local last
		local newIconSize = FuBar:GetFontSize()
		for _, v in ipairs(buttons) do
			local name = v.id
			local button = _G["MusicStreamFuFrame" .. name]
			if button then
				if self:IsShowingButton(name) then
					button:SetWidth(newIconSize)
					button:SetHeight(newIconSize)
					button:Show()

					if last then
						button:SetPoint("LEFT", last, "RIGHT", self:GetButtonSpacing(), 0)
					else
						button:SetPoint("LEFT", self.frame, "LEFT", self:GetButtonSpacing(), 0)
					end

					frameWidth = frameWidth + button:GetWidth() + self:GetButtonSpacing()
					last = button
				else
					button:Hide()
				end
			end
		end

		self.fontSize = fsize
		self.frame:SetWidth(frameWidth)
	end
end