--[[

Name: Music-Library
Revision: $Revision$
Author(s): benumbed (someone@somewhere.com)
Website: http://www.wowace.com/wiki/MusicStream
Documentation: http://www.wowace.com/wiki/MusicStream
SVN: http://svn.wowace.com/root/trunk/MusicStream
Description: A library to provide a clean way to Play Music with ID3 Tags
Dependencies: AceLibrary, AceEvent-2.0, AceHook-2.1

]]--

local MAJOR_VERSION = "Music-2.0"
local MINOR_VERSION = "$Revision: 23368 $"

if not AceLibrary then error(MAJOR_VERSION .. " requires AceLibrary.") end
if not AceLibrary:IsNewVersion(MAJOR_VERSION, MINOR_VERSION) then return end
if not AceLibrary:HasInstance("AceEvent-2.0") then error(MAJOR_VERSION .. " requires AceEvent-2.0.") end

local lib = AceLibrary("AceLocale-2.2"):new(MAJOR_VERSION)

local lib = {}
AceLibrary("AceDebug-2.0"):embed(lib)
AceLibrary("AceEvent-2.0"):embed(lib)

-- Activate a new instance of this library
function activate(self, oldLib, oldDeactivate)
	if oldLib then
		self.vars = oldLib.vars
		oldLib:UnregisterAllEvents()
	else
		self.vars = {}
	end
	
	if oldDeactivate then oldDeactivate(oldLib) end
	
	Music_LoadedTracks = {};
	Music_Tracks = {};
	local temptrack = nil;
	Music_FilePath = "Interface\\AddOns\\MusicStream\\Music\\MyMusic\\";
	Blank_FilePath = "Interface\\AddOns\\MusicStream\\";
	
	local RandomMusic = false;
	
	if type(LoadMyMusic) == "function" then
		LoadMyMusic();
	end	
	
	self:GetMusic()
	
	--self:SetDebugging(true)
	
	self:RegisterEvent("Music_Control")
end

--- ========================================================= ---
--- Music Control
--- ========================================================= ---

function lib:Music_Control(command)
	if command == "Play" then
		self:Play()
	end
	
	if command == "Stop" then
		self:Stop()
	end
	
	if command == "Prev" then
		self:PlayPrevTrack()
	end
	
	if command == "Next" then
		self:PlayNextTrack()
	end
end

--- ========================================================= ---
--- Play
--- ========================================================= ---

function lib:Play()
	if not self.vars.tracksloaded then return end

	self:GetMusic();
	
	PlayMusic(self.vars.currentfilepath);
	
	self:TriggerEvent("Music_Changed")
	self:ScheduleEvent("TrackLength", self.PlayNextTrack, self.vars.currentlength, self)
end

--- ========================================================= ---
--- Stop
--- ========================================================= ---

function lib:Stop()
	StopMusic();
	PlayMusic(Blank_FilePath.."EmptyTrack.mp3");
	
	self:CancelScheduledEvent("TrackLength")
end

--- ========================================================= ---
--- Next
--- ========================================================= ---

function lib:PlayNextTrack()
	Track = self.vars.next;
	self:Play();
end

--- ========================================================= ---
--- Prev
--- ========================================================= ---

function lib:PlayPrevTrack()
	Track = self.vars.prev;
	self:Play();
end

--- ========================================================= ---
--- Set Random Music
--- ========================================================= ---

function lib:SetRandomMusic(var)
	RandomMusic = var;
end

--- ========================================================= ---
--- Get Music
--- ========================================================= ---

function lib:GetMusic()

	if not TracksLoaded then return else self.vars.tracksloaded = TracksLoaded end

	if not(Track) then
		Track = 1;
	end
	
	if Track > getn(Music_LoadedTracks) then
		Track = 1;
	end

	if not RandomMusic then
		if Track == 1 then
			self.vars.prev = getn(Music_LoadedTracks)
			self:SetPrevTrack()
		else
			self.vars.prev = (Track - 1);
			self:SetPrevTrack()
		end
	
		if Track == getn(Music_LoadedTracks) then
			self.vars.next = 1;
			self:SetNextTrack()
		else
			self.vars.next = (Track + 1);
			self:SetNextTrack()
		end
	else
		self.vars.prev = random(1, getn(Music_LoadedTracks));
		self:SetPrevTrack()
		
		self.vars.next = random(1, getn(Music_LoadedTracks));
		self:SetNextTrack()
	end
	
	self.vars.current = Track
	self:SetCurrentTrack()
end

--- ========================================================= ---
--- Set Current Track
--- ========================================================= ---

function lib:SetCurrentTrack()
	self.vars.currentfile = Music_LoadedTracks[self.vars.current];
	
	local fileName = Music_FilePath..""..self.vars.currentfile..".mp3";
	local trackInfo = Music_Tracks[self.vars.currentfile];
	
	self.vars.currenttitle = trackInfo.title;
	self.vars.currentartist = trackInfo.artist;
	self.vars.currentalbum = trackInfo.album;
	self.vars.currentfilepath = fileName;
	self.vars.currentlength = trackInfo.length;
	self.vars.currenttrack = trackInfo.track;
	self.vars.currentgenre = trackInfo.genre;
end

--- ========================================================= ---
--- Set Next Track
--- ========================================================= ---

function lib:SetNextTrack()
	self.vars.nextfile = Music_LoadedTracks[self.vars.next];

	local fileName = Music_FilePath..""..self.vars.nextfile..".mp3";
	local trackInfo = Music_Tracks[self.vars.nextfile];

	self.vars.nexttitle = trackInfo.title;
	self.vars.nextartist = trackInfo.artist;
	self.vars.nextalbum = trackInfo.album;
	self.vars.nextfilepath = fileName;
	self.vars.nextlength = trackInfo.length;
	self.vars.nexttrack = trackInfo.track;
	self.vars.nextgenre = trackInfo.genre;
	
	test = self.vars.next
end

--- ========================================================= ---
--- Set Prev Track
--- ========================================================= ---

function lib:SetPrevTrack()
	self.vars.prevfile = Music_LoadedTracks[self.vars.prev];

	local fileName = Music_FilePath..""..self.vars.prevfile..".mp3";
	local trackInfo = Music_Tracks[self.vars.prevfile];

	self.vars.prevtitle = trackInfo.title;
	self.vars.prevartist = trackInfo.artist;
	self.vars.prevalbum = trackInfo.album;
	self.vars.prevfilepath = fileName;
	self.vars.prevlength = trackInfo.length;
	self.vars.prevtrack = trackInfo.track;
	self.vars.prevgenre = trackInfo.genre;
end

--- ========================================================= ---
--- Add Track
--- ========================================================= ---

function AddTrack(TName, _length, _title, _artist, _album, _genre)
	if not temptrack then
		temptrack = 0;
	end
	
	temptrack = temptrack + 1;
	
	Music_Tracks[TName] = { title = _title, artist = _artist, album = _album, genre = _genre, length = _length, filename = TName, track = temptrack };
	table.insert(Music_LoadedTracks, TName);
	
	TracksLoaded = true;
end

--- ========================================================= ---
--- Query Methods
--- ========================================================= ---

function lib:TracksLoaded()
	return self.vars.tracksloaded
end

function lib:Current()
	return self.vars.current
end

function lib:CurrentFile()
	return self.vars.currentfile
end

function lib:CurrentTitle()
	return self.vars.currenttitle
end

function lib:CurrentArtist()
	return self.vars.currentartist
end

function lib:CurrentAlbum()
	return self.vars.currentalbum
end

function lib:CurrentFilePath()
	return self.vars.currentfilepath
end

function lib:CurrentLength()
	return self.vars.currentlength
end

function lib:CurrentTrack()
	return self.vars.currenttrack
end

function lib:CurrentGenre()
	return self.vars.currentgenre
end

function lib:Next()
	return self.vars.next
end

function lib:NextFile()
	return self.vars.nextfile
end

function lib:NextTitle()
	return self.vars.nexttitle
end

function lib:NextArtist()
	return self.vars.nextartist
end

function lib:NextAlbum()
	return self.vars.nextalbum
end

function lib:NextFilePath()
	return self.vars.nextfilepath
end

function lib:NextLength()
	return self.vars.nextlength
end

function lib:NextTrack()
	return self.vars.nexttrack
end

function lib:NextGenre()
	return self.vars.nextgenre
end

function lib:Prev()
	return self.vars.prev
end

function lib:PrevFile()
	return self.vars.prevfile
end

function lib:PrevTitle()
	return self.vars.prevtitle
end

function lib:PrevArtist()
	return self.vars.prevartist
end

function lib:PrevAlbum()
	return self.vars.prevalbum
end

function lib:PrevFilePath()
	return self.vars.prevfilepath
end

function lib:PrevLength()
	return self.vars.prevlength
end

function lib:PrevTrack()
	return self.vars.prevtrack
end

function lib:PrevGenre()
	return self.vars.prevgenre
end

--- ========================================================= ---
--- Welcome to AceLibrary
--- ========================================================= ---
AceLibrary:Register(lib, MAJOR_VERSION, MINOR_VERSION, activate)