﻿--[[

Name: MusicStrem Random Track
Revision: $Revision: 23844 $
Date: $Date: 2007-01-07 03:01:22 -0500 (Sun, 07 Jan 2007) $
Author(s): benumbed (someone@somewhere.com)
Website: http://www.wowace.com/wiki/MusicStream
Documentation: http://www.wowace.com/wiki/MusicStream
SVN: http://svn.wowace.com/root/trunk/MusicStream
Description: A module to change the RandomMusic variable in the Music Library.
Dependencies: Ace2, MusicStream

]]--

--- ========================================================= ---
--- Revision
--- ========================================================= ---

local Revision = tonumber(string.sub("$Revision: 23844 $", 12, -3))
if not Revision then
	Revision = "2.0"
end

--- ========================================================= ---
--- Add Locale
--- ========================================================= ---
local L = AceLibrary("AceLocale-2.2"):new("RandomTrack")

L:RegisterTranslations("enUS", function() return {
    ["Random Track"] = true,
    ["Toggle saving Track last played"] = true,
	
    ["Toggle"] = true,
    ["Toggle the Module On and Off"] = true,
} end)

--- ========================================================= ---
--- Add Music Library
--- ========================================================= ---
local Music = AceLibrary("Music-2.0")

--- ========================================================= ---
--- Register New Module
--- ========================================================= ---

MusicStreamRandomTrack = MusicStream:NewModule("RandomTrack")

--- ========================================================= ---
--- On Initialize
--- ========================================================= ---

function MusicStreamRandomTrack:OnInitialize()
    self.db = MusicStream:AcquireDBNamespace("RandomTrack")
	
    MusicStreamRandomTrack.defaultDB = {
        on = false,
    }
	
	MusicStream:RegisterDefaults("RandomTrack", "profile", opts or MusicStreamRandomTrack.defaultDB or {})
	
    MusicStream:ToggleModuleActive("RandomTrack", self.db.profile.on)
    
	if (self.db.profile.on == true) then
		if (self.db.profile.track) then
			Track = self.db.profile.track;
		end
	end
	
	local bluecolor = "|cFF0099CC"
	local lbluecolor = "|cFF77BBFF"
	
    MusicStream.Options.args.settings.args.RandomTrack = {
        name = L["Random Track"],
        desc = L["Toggle saving Track last played"],
		order = 101,
        type = "group",
        args = {
		     header = {
				type = "header",
				name = bluecolor..L["Random Track"].."  |c88888888"..Revision,
				icon = "Interface\\Addons\\MusicStream\\icon.tga",
				iconHeight = 14,
				iconWidth = 14,
				order = 1
            },
			blank = {
       			type = "header",
				order = 2,
			},
            toggle = {
                name = L["Toggle"],
                desc = L["Toggle the Module On and Off"],
				order = 200,
                type = "toggle",
                get = function() return self.db.profile.on end,
                set = function() self.db.profile.on = MusicStream:ToggleModuleActive("RandomTrack") end
            }
        }
    }
end

--- ========================================================= ---
--- On Enable
--- ========================================================= ---

function MusicStreamRandomTrack:OnEnable()
	Music:SetRandomMusic(true);
	Music:GetMusic()
end

--- ========================================================= ---
--- On Disable
--- ========================================================= ---

function MusicStreamRandomTrack:OnDisable()
	Music:SetRandomMusic(false);
	Music:GetMusic()
end

--- ========================================================= ---
--- Reset DB
--- ========================================================= ---

local function copyTable(to, from)
	setmetatable(to, nil)
	for k,v in pairs(from) do
		if type(k) == "table" then
			k = copyTable({}, k)
		end
		if type(v) == "table" then
			v = copyTable({}, v)
		end
		to[k] = v
	end
	setmetatable(to, from)
	return to
end

function MusicStreamRandomTrack:ResetDB()
	copyTable(self.db.profile, MusicStreamRandomTrack.defaultDB)
end